

#local
#code VCC_LONG
#include "vars32.s"


; -----------------------------------------
; 	32 bit arithmetics
; -----------------------------------------


; ________________________________
; add 32 bit values
;
; ADD32 ( n1.l n2.l -- n3.l )

ADD32::	pop	hl		; hl = n2.hi, de = n2.lo, stack=n1.lo,n1.hi
	ex	hl,(sp)		; hl = n1.lo, de = n2.lo, stack=n2.hi,n1.hi
	add	hl,de		; hl = n3.lo,             stack=n2.hi,n1.hi
	pop	de		; hl = n3.lo, de = n2.hi, stack=n1.hi
	ex	hl,(sp)		; hl = n1.hi, de = n2.hi, stack=n3.lo
	adc	hl,de		; hl = n3.hi,             stack=n3.lo
	pop	de		; hl = n3.hi, de = n3.lo
	jr	push_hl_next	; de = n3.lo,             stack=n3.hi

; ________________________________
; subtract 32 bit values n1 - n2
;
; SUB32 ( n1.l n2.l -- n3.l )

SUB32::	and	a
	pop	hl		; hl = n2.hi, de = n2.lo, stack=n1.lo,n1.hi
	ex	hl,(sp)		; hl = n1.lo, de = n2.lo, stack=n2.hi,n1.hi
	sbc	hl,de		; hl = n3.lo,             stack=n2.hi,n1.hi
	pop	de		; hl = n3.lo, de = n2.hi, stack=n1.hi
	ex	hl,(sp)		; hl = n1.hi, de = n2.hi, stack=n3.lo
	sbc	hl,de		; hl = n3.hi,             stack=n3.lo
	pop	de		; hl = n3.hi, de = n3.lo
	jr	push_hl_next	; de = n3.lo,             stack=n3.hi

; ________________________________
; reset bits in long word
;
; AND32 ( n32 n32 -- n32 )

AND32::	pop	hl		; de=n2lo, hl=n2hi, stack:n1hi n1lo
	ex	hl,(sp)		; de=n2lo, hl=n1lo, stack:n1hi n2hi
	ld	a,h
	and 	d
	ld	h,a
	ld	a,l
	and	e
	ld	l,a		; hl=reslo, stack:n1hi n2hi
	pop	de		; de=n2hi, hl=reslo, stack:n1hi 
	ex	hl,(sp)		; de=n2hi, hl=n1hi, stack: reslo
	ld	a,h
	and 	d
	ld	h,a
	ld	a,l
	and	e
	ld	l,a		; hl=reshi, stack:reslo
	pop	de
	jr	push_hl_next	

; ________________________________
; set bits in long word
;
; OR32 ( n32 n32 -- n32 )

OR32::	pop	hl		; de=n2lo, hl=n2hi, stack:n1hi n1lo
	ex	hl,(sp)		; de=n2lo, hl=n1lo, stack:n1hi n2hi
	ld	a,h
	or 	d
	ld	h,a
	ld	a,l
	or	e
	ld	l,a		; hl=reslo, stack:n1hi n2hi
	pop	de		; de=n2hi, hl=reslo, stack:n1hi 
	ex	hl,(sp)		; de=n2hi, hl=n1hi, stack: reslo
	ld	a,h
	or 	d
	ld	h,a
	ld	a,l
	or	e
	ld	l,a		; hl=reshi, stack:reslo
	pop	de
	jr	push_hl_next	

; ________________________________
; toggle bits in long word
;
; XOR32 ( n32 n32 -- n32 )

XOR32::	pop	hl		; de=n2lo, hl=n2hi, stack:n1hi n1lo
	ex	hl,(sp)		; de=n2lo, hl=n1lo, stack:n1hi n2hi
	ld	a,h
	xor 	d
	ld	h,a
	ld	a,l
	xor	e
	ld	l,a		; hl=reslo, stack:n1hi n2hi
	pop	de		; de=n2hi, hl=reslo, stack:n1hi 
	ex	hl,(sp)		; de=n2hi, hl=n1hi, stack: reslo
	ld	a,h
	xor 	d
	ld	h,a
	ld	a,l
	xor	e
	ld	l,a		; hl=reshi, stack:reslo
	pop	de
	jr	push_hl_next	

; ________________________________
; shift long value left once
;
; SL132 ( n32 -- n32 )

SL132::	ex	de,hl
	add	hl,hl		; SL the low word
	ex	de,hl
	pop	hl
	adc	hl,hl		; SL the high word
	jr	push_hl_next

; ________________________________
; shift unsigned long value right once
;
; SR1u32 ( n32 -- n32 )

SR1u32::pop	hl
	srl	h		; SR the high word
	rr	l
	rr	d		; SR the low word
	rr	e
	jr	push_hl_next

; ________________________________
; shift signed long value right once
;
; SR1s32 ( n32 -- n32 )

SR1s32::pop	hl
	sra	h		; SR the high word
	rr	l
	rr	d		; SR the low word
	rr	e
	jr	push_hl_next

; ________________________________
; calculate absolute value of signed long value
; return value is uint32, because abs($80000000) = $80000000
;
; ABS32 ( n32 -- n32 )

ABS32::	pop	af
	push	af
	and	a
	jp	p,next		; positive
	;jr	NEG32

; ________________________________
; negate
;
; NEG32 ( n32 -- n32 )

NEG32::	xor	a
	ld	h,a
	ld	l,a
	sbc	hl,de
	pop	de
	push	hl
	ld	h,a
	ld	l,a
	sbc	hl,de
	pop	de
	jp	push_hl_next

; ________________________________
; get value from long variable
; with post-increment
;
; PEEKPP32 ( &n32 -- n32 )

PEEKPP32::
	ld	hl,3
	add	hl,de		; hl -> var.hhi
	
	ld	e,(hl)
	ld	d,(--hl)
	push	de		; stack: old value.hi
	
	ld	e,(--hl)
	ld	d,(--hl)	; d = old value.lo, hl -> var
	
	jr	incr

; ________________________________
; get value from long variable
; with post-decrement
;
; PEEKMM32 ( &n32 -- n32 )

PEEKMM32::
	ld	hl,3
	add	hl,de		; hl -> var.hhi
	
	ld	e,(hl)
	ld	d,(--hl)
	push	de		; stack: old value.hi
	
	ld	e,(--hl)
	ld	d,(--hl)	; d = old value.lo, hl -> var
	
	jr	decr

; ________________________________
; get value from long variable
; with pre-decrement
;
; MMPEEK32 ( &n32 -- n32 )

MMPEEK32::
	ld	hl,de		; hl->var, preserve de->var
	
	dec	(hl)
	ld	a,(hl)
	inc	a
	jp	nz,ld_top32_xhl_next
	inc	hl
	
	cp	a,(hl)
	jr	nz,2$
	dec	(hl++)
	
	cp	a,(hl)
	jr	nz,2$
	dec	(hl++)
	
2$:	dec	(hl)	
	jr	PEEK32

; ________________________________
; get value from long variable
; with pre-increment
;
; PPPEEK32 ( &n32 -- n32 )

PPPEEK32::
	ld	hl,de		; hl->var, preserve de->var
	
	inc	(hl)
	jp	nz,ld_top32_xhl_next
	inc	hl

	inc	(hl)
	jr	nz,PEEK32
	inc	hl

	inc	(hl)
	jr	nz,PEEK32
	inc	hl

	inc	(hl)	
	;jr	PEEK32

; ________________________________
; get value from long variable
;
; PEEK32 ( &n32 -- n32 )

PEEK32::
	ex	hl,de		; hl -> var
	jp	ld_top32_xhl_next

; ________________________________
; store long value in variable
;
; POKE32 ( n32 &var -- )

POKE32::
	ex	hl,de		; hl -> var
	pop	de
poke32:	jp	ld_xhl_top32_next

; ________________________________
; store long value in variable
;
; NPOKE32 ( &var n32 -- )

NPOKE32::
	pop	hl		; de = n.lo, hl=n.hi
	ex	hl,(sp)		; de = n.lo, hl->var, stack:n.hi
	jr	poke32
	
; ________________________________
; increment long variable
;
; PPSET32 ( &n32 -- )

PPSET32::
	ex	de,hl		; hl->var
	pop	de		; de = new TOP
	
incr:	inc	(hl)
	jp	nz,next
	inc	hl

	inc	(hl)
	jr	nz,1$
	inc	hl

	inc	(hl)
	jr	nz,1$
	inc	hl

	inc	(hl)
1$:	jp	next

; ________________________________
; decrement long variable
;
; MMSET32 ( &n32 -- )

MMSET32::
	ex	de,hl		; hl->var
	pop	de		; de = new TOP
decr:
	dec	(hl)
	ld	a,(hl)
	inc	a
	jp	nz,next
	inc	hl
	
	cp	a,(hl)
	jr	nz,2$
	dec	(hl++)
	
	cp	a,(hl)
	jr	nz,2$
	dec	(hl++)
	
2$:	dec	(hl)	
	jp	next

; ________________________________
; mask 32 bit variable with AND
;
; ANDGL32 ( n32 &var -- )

ANDGL32::
	ex	hl,de		; hl -> var

	pop	de		; de = n.lo

	ld	a,(hl)
	and	e
	ld	(hl++),a

	ld	a,(hl)
	and	d
	ld	(hl++),a

	pop	de		; de = n.hi

	ld	a,(hl)
	and	e
	ld	(hl++),a

	ld	a,(hl)
	and	d
	ld	(hl),a

	jp	pop_de_next

; ________________________________
; mask 32 bit variable with OR
;
; ORGL32 ( n32 &var -- )

ORGL32::
	ex	hl,de		; hl -> var

	pop	de		; de = n.lo

	ld	a,(hl)
	or	e
	ld	(hl++),a

	ld	a,(hl)
	or	d
	ld	(hl++),a

	pop	de		; de = n.hi

	ld	a,(hl)
	or	e
	ld	(hl++),a

	ld	a,(hl)
	or	d
	ld	(hl),a

	jp	pop_de_next

; ________________________________
; mask 32 bit variable with XOR
;
; XORGL32 ( n32 &var -- )

XORGL32::
	ex	hl,de		; hl -> var

	pop	de		; de = n.lo

	ld	a,(hl)
	xor	e
	ld	(hl++),a

	ld	a,(hl)
	xor	d
	ld	(hl++),a

	pop	de		; de = n.hi

	ld	a,(hl)
	xor	e
	ld	(hl++),a

	ld	a,(hl)
	xor	d
	ld	(hl),a

	jp	pop_de_next

; ________________________________
; add 32 bit value to 32 bit variable
;
; ADDGL32 ( n32 &var -- )

ADDGL32::
	ex	hl,de		; hl -> var

	pop	de		; de = n.lo

	ld	a,(hl)
	add	e
	ld	(hl++),a

	ld	a,(hl)
	adc	d
	ld	(hl++),a

	pop	de		; de = n.hi

	ld	a,(hl)
	adc	e
	ld	(hl++),a

	ld	a,(hl)
	adc	d
	ld	(hl),a

	jp	pop_de_next

; ________________________________
; subtract from long variable
;
; SUBGL32 ( n32 &var -- )

SUBGL32::
	ex	hl,de		; hl -> var
	
	pop	de		; de = n.lo

	ld	a,(hl)
	sub	e
	ld	(hl++),a
	
	ld	a,(hl)
	sbc	d
	ld	(hl++),a

	pop	de		; de = n.hi

	ld	a,(hl)
	sbc	e
	ld	(hl++),a
	
	ld	a,(hl)
	sbc	d
	ld	(hl),a

	jp	pop_de_next	


; ________________________________
; shift signed long variable right N bits
;
; SRGLs32 ( n16 &var -- )

SRGLs32::
	ex	hl,de		; hl -> var
	pop	de		; e = shift
	ld	d,e		; d = shift
	push	hl		; stack: ->var
	push	de		; stack: ->var, shift
	 
	ld	e,(hl++)
	ld	d,(hl++)
	ld	a,(hl++)
	ld	h,(hl)
	ld	l,a		; hlde = var

	pop	af		; a = shift

	call	srs32		; hlde = hlde >> a
	
	ex	hl,(sp)		; hl->var, de=n.lo, stack:n.hi	
	jp	ld_xhl_top32_next

; ________________________________
; shift unsigned long variable right N bits
;
; SRGLu32 ( n16 &var -- )

SRGLu32::
	ex	hl,de		; hl -> var
	pop	de		; e = shift
	ld	d,e		; d = shift
	push	hl		; stack: ->var
	push	de		; stack: ->var, shift
	 
	ld	e,(hl++)
	ld	d,(hl++)
	ld	a,(hl++)
	ld	h,(hl)
	ld	l,a		; hlde = var

	pop	af		; a = shift

	call	sru32		; hlde = hlde >> a
	
	ex	hl,(sp)		; hl->var, de=n.lo, stack:n.hi
	jp	ld_xhl_top32_next

; ________________________________
; shift long variable left N bits
;
; SLGL32 ( n &var -- n32 )

SLGL32::
	ex	hl,de		; hl -> var
	pop	de		; e = shift
	ld	d,e		; d = shift
	push	hl		; stack: ->var
	push	de		; stack: ->var, shift
	 
	ld	e,(hl++)
	ld	d,(hl++)
	ld	a,(hl++)
	ld	h,(hl)
	ld	l,a		; hlde = var

	pop	af		; a = shift

	call	sl32		; hlde = hlde >> a
	
	ex	hl,(sp)		; hl->var, de=n.lo, stack:n.hi
	jp	ld_xhl_top32_next

; ________________________________
; shift signed long value right by N bits
;
; SRs32 ( n32 n -- n32 )

SRs32::	ld	a,e		; a = shift
	pop	de		; de = n32.lo
	ld	hl,push_hl_next
	ex	hl,(sp)		; hlde = n32
	;jp	srs32

; ________________________________
; shift signed long value right by A bits
;
; HLDE >>= A
;
srs32:	and	0x1f	
	
1$:	ret	z
	cp	8               ; set carry flag if a < 8
	jr 	c,3$        	; no more full bytes to shift
	ld 	e,d
	ld 	d,l
	ld 	l,h
	ld 	h,0		; hlde = hlde >> 8            
	
	bit	7,l		; neg. number?
	jr	z,2$		; no
	dec	h		; yes => h := 0xff
2$:	sub	8
	jr 	1$

3$:	sra	h
	rr 	l
	rr 	d
	rr 	e
	dec	a               ; set zero flag if a == 0
	jr 	nz,3$
	ret			; result in hlde
	

; ________________________________
; shift unsigned long value right by N bits 
;
; SRu32 ( n32 n -- n32 )

SRu32::	ld	a,e		; a = shift
	pop	de		; de = n32.lo
	ld	hl,push_hl_next
	ex	hl,(sp)		; hlde = n32
	;jp	sru32

; ________________________________
; shift unsigned long value right by A bits
;
; HLDE >>= A
;
sru32:	and	0x1f	
	
1$:	ret	z
	cp	8               ; set carry flag if a < 8
	jr 	c,3$        	; no more full bytes to shift
	ld 	e,d
	ld 	d,l
	ld 	l,h
	ld 	h,0		; hlde = hlde >> 8             
	sub	8
	jr 	1$

3$:	srl	h
	rr 	l
	rr 	d
	rr 	e
	dec	a               ; set zero flag if a == 0
	jr 	nz,3$
	ret			; result in hlde
	
; ________________________________
; shift long value left N bits 
;
; SL32 ( n32 n -- n32 )

SL32::	ld	a,e		; a = shift
	pop	de		; de = n32.lo
	ld	hl,push_hl_next
	ex	hl,(sp)		; hlde = n32
	;jp	sl32

; ________________________________
; shift long value left by A bits
;
; HLDE <<= A
;
sl32:	and	0x1f	

1$:	ret	z
	cp	8               ; set carry flag if a < 8
	jr 	c,3$        	; no more full bytes to shift
	ld	h,l
	ld	l,d
	ld	d,e
	ld	e,0		; hlde = hlde << 8
	sub	8		; count -= 8
	jr 	1$

3$:	sla	e
	rl 	d
	rl 	l
	rl 	h
	dec	a
	jr 	nz,3$
	ret			; result in hlde
	

; ________________________________
; complement
;
; CPL32 ( n32 -- n32 )

CPL32::	pop	hl
	ld	a,h
	cpl
	ld	h,a
	ld	a,l
	cpl	
	ld	l,a
	push	hl
	
	jp	CPL	; könnten wir direkt dorthin platzieren

; ________________________________
; boolean negation: !n
;
; NOT32 ( n32 -- n )

NOT32::	pop	hl
	ld	a,d
	or	e
	or	h
	or	l
	ld	de,0
	jp	nz,next
	inc	e
	jp	next

; ________________________________
; calculate sign of signed long int
;
; SIGN32 ( n32 -- n )

SIGN32::
	pop	hl
	ld	a,h
	add	a
	jr	c,1$	; -1
	or	l
	or	d
	or	e
	jr	z,2$	; 0

	ld	de,1
2$:	jp	next

1$:	ld	de,-1
	jp	next

; ________________________________
; convert long int to boolean
;
; BOOL32 ( n32 -- n )

BOOL32::
	pop	hl
	ld	a,d
	or	e
	or	h
	or	l
	jp	z,next
	ld	de,1
	jp	next

; ________________________________
; calc msbit
;
; MSBIT32 ( n32 -- n )

MSBIT32::
	pop	hl		; hl = NN.hi
	ld	a,h
	or	l
	jp	z,MSBIT		; NN.hi = 0 -> calc msbit(NN.lo)

    ; calc msbit(NN.hi) and add 16:

	ex	hl,de
	push	bc
	virtual
	dw	MSBIT
	dw	NATIVE
	pop	bc
	set	4,e
	jp	next

; ________________________________
EXTENDs32::
	ld	a,d
	add	a
	sbc	hl,hl
	jp	push_hl_next
	
; ________________________________
EXTENDu32::
	ld	hl,0
	jp	push_hl_next


; ________________________________
#include "mul32.s"
#include "div32.s"
	


; ______________________________________
; compare two values: n1 == n2
; either signed - signed or
; unsigned - unsigned
;
; EQ32 ( n32 n32 -- f )

EQ32::	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de
	pop	hl
	pop	de
	jp	nz,return_false
	sbc	hl,de
	jp	nz,return_false
	jp	return_true

; ______________________________________
; compare two values: n1 != n2
; either signed - signed or
; unsigned - unsigned
;
; NE32 ( n32 n32 -- f )

NE32::	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de
	pop	hl
	pop	de
	jp	nz,return_true
	sbc	hl,de
	jp	nz,return_true
	jp	return_false

; ______________________________________
; compare two signed values: n1 < n2
;
; LTs32 ( n32 n32 -- f )

LTs32::	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	xor	h
	jp	m,drop2_true_if_hl_pos	; different sign => n1 LT n2 is true if n2 pos
	db	opcode(cp a,N)		; jr ltu32       => else do unsigned compare
	
; ______________________________________
; compare two unsigned values: n1 < n2
;
; LTu32 ( n32 n32 -- f )

LTu32::	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
ltu32:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	de		; de=n2hi
	pop	hl		; hl=n1hi
	sbc	hl,de		; cy => n1hi - n2hi - cy < 0 => n1 < n2 => LT 
	jp	ld_de_cy_next

; ______________________________________
; compare two signed values: n1 ≥ n2
;
; GEs32 ( n32 n32 -- f )

GEs32::	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	xor	h
	jp	m,drop2_true_if_hl_neg	; different sign => n1 ≥ n2 is true if n2 neg
	db	opcode(cp a,N)		; jr geu32       => else do unsigned compare
	
; ______________________________________
; compare two unsigned values: n1 ≥ n2
;
; GEu32 ( n32 n32 -- f )

GEu32::	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
geu32:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	de		; de=n2hi
	pop	hl		; hl=n1hi
	sbc	hl,de		; cy => n1hi - n2hi - cy < 0 => n1 < n2 => LT, not GE
	jp	ld_de_nc_next

; ______________________________________
; compare two unsigned values: n1 > n2
;
; GTu32 ( n32 n32 -- f )

GTu32::	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
gtu32:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	ex	hl,de
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	hl		; hl=n2hi
	pop	de		; de=n1hi
	sbc	hl,de		; cy => n2hi - n1hi - cy < 0 => n2 < n1 => GT 
	jp	ld_de_cy_next

; ______________________________________
; compare two unsigned values: n1 ≤ n2
;
; LEu32 ( n32 n32 -- f )

LEu32::	pop	hl		; hl=n2hi, de=n2lo, stack:n1hi n1lo
leu32:	ex	hl,(sp)		; hl=n1lo, de=n2lo, stack:n1hi n2hi
	ex	hl,de
	xor	a		; cy=0, a=0
	sbc	hl,de		; cy=cy, stack:n1hi n2hi
	pop	hl		; hl=n2hi
	pop	de		; de=n1hi
	sbc	hl,de		; cy => n2hi - n1hi - cy < 0 => n2 < n1 => GT 
	jp	ld_de_nc_next

; ______________________________________
; compare two signed values: n1 > n2
;
; GTs32 ( n32 n32 -- f )

GTs32::	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	xor	h
	jp	p,gtu32		; same sign => unsigned compare
	;jp	m,drop2_true_if_hl_neg	; different sign => n1 > n2 is true if n2 neg

drop2_true_if_hl_neg::
	pop	af
	pop	af
	xor	a		; a=0 req. by ld_de_cy_next
	add	hl,hl		; cy = hl.vz
	jp	ld_de_cy_next	; result true if hl<0
	
; ______________________________________
; compare two signed values: n1 ≤ n2
;
; LEs32 ( n32 n32 -- f )

LEs32::	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	xor	h
	jp	p,leu32		; same sign => unsigned compare
	;jp	m,drop2_true_if_hl_pos	; different sign => n1 ≤ n2 is true if n2 pos

drop2_true_if_hl_pos::
	pop	af
	pop	af
	xor	a		; a=0 req. by ld_de_nc_next
	add	hl,hl		; cy = hl.vz
	jp	ld_de_nc_next	; result true if hl≥0

	



; -----------------------------------------
; 	Mixed Signedness Compare
; -----------------------------------------

; ______________________________________
; compare two long values with mixed signedness
; either signed - unsigned or vice versa
; 
; EQx32 ( n32 n32 -- f )

EQx32::	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de		
	pop	hl
	pop	de
	jp	nz,return_false	; low words NE
	sbc	hl,de
	jp	nz,return_false	; high words NE
	
	bit	7,d		; different sign?
	jp	nz,return_false	; => NE		TODO: simply JP PO,false ?	
	jp	return_true

; ______________________________________
; compare two long values with mixed signedness
; either signed - unsigned or vice versa
; 
; NEx32 ( n32 n32 -- f )

NEx32::	pop	hl		; hl = n2hi, de = n2lo, stack: n1hi n1lo
	ex	hl,(sp)		; hl = n1lo, de = n2lo, stack: n1hi n2hi
	and	a
	sbc	hl,de		
	pop	hl
	pop	de
	jp	nz,return_true	; low words NE
	sbc	hl,de
	jp	nz,return_true	; high words NE
	
	bit	7,d		; different sign?
	jp	nz,return_true	; => NE		TODO: simply JP PO,true ?	
	jp	return_false

; ______________________________________
; compare two values with mixed signedness u32 > i32
;
; GTus32 ( u32 i32 -- f )

GTus32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,gtu32		; no
	jr	drop2_true_if_hl_neg	; yes
	
; ______________________________________
; compare two values with mixed signedness i32 > u32
;
; GTsu32 ( i32 u32 -- f )

GTsu32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,gtu32		; no
	jr	drop2_true_if_hl_pos	; yes
	
; ______________________________________
; compare two values with mixed signedness i32 < u32
;
; LTsu32 ( i32 u32 -- f )

LTsu32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,ltu32		; no
	jr	drop2_true_if_hl_pos	; yes
	
; ______________________________________
; compare two values with mixed signedness u32 < i32
;
; LT32us ( u32 i32 -- f )

LTus32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,ltu32		; no
	jr	drop2_true_if_hl_neg	; yes
	
; ______________________________________
; compare two values with mixed signedness u32 ≥ i32
;
; GEus32 ( u32 i32 -- f )

GEus32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,geu32		; no
	jp	drop2_true_if_hl_neg	; yes
	
; ______________________________________
; compare two values with mixed signedness i32 ≥ u32
;
; GEsu32 ( i32 u32 -- f )

GEsu32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,geu32		; no
	jr	drop2_true_if_hl_pos	; yes
	
; ______________________________________
; compare two values with mixed signedness i32 ≤ u32
;
; LEsu32 ( i32 u32 -- f )

LEsu32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,leu32		; no
	jp	drop2_true_if_hl_pos	; yes
	
; ______________________________________
; compare two values with mixed signedness u32 ≤ i32
;
; LEus32 ( u32 i32 -- f )

LEus32::
	ld	hl,5		; de = n2lo, stack: n1hi n1lo n2hi
	add	hl,sp
	ld	a,(hl)		; a = n1hhi
	pop	hl		; hl = n2hi
	or	h		; u16≥0x8000 or i16<0 ?
	jp	p,leu32		; no
	jp	drop2_true_if_hl_neg	; yes
	

; ________________________________
; swap two long variables
;
; SWAPVARS ( &var &var -- )

SWAPVARS32::
	pop	hl
	
	ld	a,(de)
	ld	e,(hl)
	ld	(hl++),a
	ld	a,e
	ld	(de++),a
	
	ld	a,(de)
	ld	e,(hl)
	ld	(hl++),a
	ld	a,e
	ld	(de++),a

	jp	swapvars


	
	




#endlocal



